@@ -1,11 +1,25 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
+
 from fnmatch import fnmatchcase
 
 from trac.config import Option
 from trac.core import *
 from trac.perm import IPermissionPolicy
 
-revision = "$Rev: 11490 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.0.1/sample-plugins/permissions/public_wiki_policy.py $"
+revision = "$Rev: 12500 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/permissions/public_wiki_policy.py $"
 
 class PublicWikiPolicy(Component):
     """Allow public access to some wiki pages.
@@ -60,4 +74,3 @@ class PublicWikiPolicy(Component):
             if action.startswith('WIKI_'):
                 return True
                 # this policy ''may'' grant permissions on some wiki pages
-
