@@ -18,15 +18,19 @@ import os
 import re
 import weakref
 
-from trac.config import ListOption
-from trac.core import *
+from genshi.builder import tag
+
+from trac.config import ConfigurationError, ListOption
+from trac.core import Component, TracError, implements
 from trac.db.api import IDatabaseConnector
 from trac.db.util import ConnectionWrapper, IterableCursor
 from trac.util import get_pkginfo, getuser
-from trac.util.translation import _
+from trac.util.translation import _, tag_
 
 _like_escape_re = re.compile(r'([/_%])')
 
+_glob_escape_re = re.compile(r'[*?\[]')
+
 try:
     import pysqlite2.dbapi2 as sqlite
     have_pysqlite = 2
@@ -179,7 +183,10 @@ class SQLiteConnector(Component):
                 self._extensions.append(extpath)
         params['extensions'] = self._extensions
         if path == ':memory:':
-            if not self.memory_cnx:
+            try:
+                self.memory_cnx.cursor()
+            except (AttributeError, sqlite.DatabaseError):
+                # memory_cnx is None or database connection closed.
                 self.memory_cnx = SQLiteConnection(path, log, params)
             return self.memory_cnx
         else:
@@ -200,16 +207,21 @@ class SQLiteConnector(Component):
             if isinstance(path, unicode): # needed with 2.4.0
                 path = path.encode('utf-8')
             # this direct connect will create the database if needed
-            cnx = sqlite.connect(path,
+            cnx = sqlite.connect(path, isolation_level=None,
                                  timeout=int(params.get('timeout', 10000)))
+            cursor = cnx.cursor()
+            _set_journal_mode(cursor, params.get('journal_mode'))
+            _set_synchronous(cursor, params.get('synchronous'))
+            cnx.isolation_level = 'DEFERRED'
         else:
             cnx = self.get_connection(path, log, params)
-        cursor = cnx.cursor()
+            cursor = cnx.cursor()
         if schema is None:
             from trac.db_default import schema
         for table in schema:
             for stmt in self.to_sql(table):
                 cursor.execute(stmt)
+        cursor.close()
         cnx.commit()
 
     def to_sql(self, table):
@@ -255,20 +267,21 @@ class SQLiteConnection(ConnectionWrapper):
                              and sqlite.version_info >= (2, 5, 0)
 
     def __init__(self, path, log=None, params={}):
-        assert have_pysqlite > 0
+        if have_pysqlite == 0:
+            raise TracError(_("Cannot load Python bindings for SQLite"))
         self.cnx = None
         if path != ':memory:':
             if not os.access(path, os.F_OK):
-                raise TracError(_('Database "%(path)s" not found.', path=path))
+                raise ConfigurationError(_('Database "%(path)s" not found.',
+                                           path=path))
 
             dbdir = os.path.dirname(path)
             if not os.access(path, os.R_OK + os.W_OK) or \
                    not os.access(dbdir, os.R_OK + os.W_OK):
-                raise TracError(
-                    _('The user %(user)s requires read _and_ write '
-                      'permissions to the database file %(path)s '
-                      'and the directory it is located in.',
-                      user=getuser(), path=path))
+                raise ConfigurationError(tag_(
+                    "The user %(user)s requires read _and_ write permissions "
+                    "to the database file %(path)s and the directory it is "
+                    "located in.", user=tag.tt(getuser()), path=tag.tt(path)))
 
         self._active_cursors = weakref.WeakKeyDictionary()
         timeout = int(params.get('timeout', 10.0))
@@ -277,6 +290,7 @@ class SQLiteConnection(ConnectionWrapper):
         if isinstance(path, unicode): # needed with 2.4.0
             path = path.encode('utf-8')
         cnx = sqlite.connect(path, detect_types=sqlite.PARSE_DECLTYPES,
+                             isolation_level=None,
                              check_same_thread=sqlite_version < (3, 3, 1),
                              timeout=timeout)
         # load extensions
@@ -287,6 +301,11 @@ class SQLiteConnection(ConnectionWrapper):
                 cnx.load_extension(ext)
             cnx.enable_load_extension(False)
 
+        cursor = cnx.cursor()
+        _set_journal_mode(cursor, params.get('journal_mode'))
+        _set_synchronous(cursor, params.get('synchronous'))
+        cursor.close()
+        cnx.isolation_level = 'DEFERRED'
         ConnectionWrapper.__init__(self, cnx, log)
 
     def cursor(self):
@@ -312,6 +331,31 @@ class SQLiteConnection(ConnectionWrapper):
     def concat(self, *args):
         return '||'.join(args)
 
+    def drop_table(self, table):
+        cursor = self.cursor()
+        if sqlite_version < (3, 7, 6):
+            # SQLite versions at least between 3.6.21 and 3.7.5 have a
+            # buggy behavior with DROP TABLE IF EXISTS (#12298)
+            try:
+                cursor.execute("DROP TABLE " + self.quote(table))
+            except sqlite.OperationalError: # "no such table"
+                pass
+        else:
+            cursor.execute("DROP TABLE IF EXISTS " + self.quote(table))
+
+    def get_column_names(self, table):
+        cursor = self.cursor()
+        cursor.execute("PRAGMA table_info(%s)" % self.quote(table))
+        return [row[1] for row in cursor]
+
+    def get_last_id(self, cursor, table, column='id'):
+        return cursor.lastrowid
+
+    def get_table_names(self):
+        rows = self.execute("""
+            SELECT name FROM sqlite_master WHERE type='table'""")
+        return [row[0] for row in rows]
+
     def like(self):
         """Return a case-insensitive LIKE clause."""
         if sqlite_version >= (3, 1, 0):
@@ -325,14 +369,49 @@ class SQLiteConnection(ConnectionWrapper):
         else:
             return text
 
+    def prefix_match(self):
+        """Return a case sensitive prefix-matching operator."""
+        return 'GLOB %s'
+
+    def prefix_match_value(self, prefix):
+        """Return a value for case sensitive prefix-matching operator."""
+        return _glob_escape_re.sub(lambda m: '[%s]' % m.group(0), prefix) + '*'
+
     def quote(self, identifier):
         """Return the quoted identifier."""
-        return "`%s`" % identifier.replace('`', '``')
-
-    def get_last_id(self, cursor, table, column='id'):
-        return cursor.lastrowid
+        return _quote(identifier)
 
     def update_sequence(self, cursor, table, column='id'):
         # SQLite handles sequence updates automagically
         # http://www.sqlite.org/autoinc.html
         pass
+
+
+def _quote(identifier):
+    return "`%s`" % identifier.replace('`', '``')
+
+
+def _set_journal_mode(cursor, value):
+    if not value:
+        return
+    value = value.upper()
+    if value == 'OFF':
+        raise TracError(_("PRAGMA journal_mode `%(value)s` cannot be used in "
+                          "SQLite", value=value))
+    cursor.execute('PRAGMA journal_mode = %s' % _quote(value))
+    row = cursor.fetchone()
+    if not row:
+        raise TracError(_("PRAGMA journal_mode isn't supported by SQLite "
+                          "%(version)s", version=sqlite_version_string))
+    if (row[0] or '').upper() != value:
+        raise TracError(_("PRAGMA journal_mode `%(value)s` isn't supported by "
+                          "SQLite %(version)s",
+                          value=value, version=sqlite_version_string))
+
+
+def _set_synchronous(cursor, value):
+    if not value:
+        return
+    if value.isdigit():
+        value = str(int(value))
+    cursor.execute('PRAGMA synchronous = %s' % _quote(value))
