@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2011 Edgewall Software
+# Copyright (C) 2003-2014 Edgewall Software
 # Copyright (C) 2003-2007 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -25,8 +25,9 @@ from urlparse import urlsplit
 
 from trac import db_default
 from trac.admin import AdminCommandError, IAdminCommandProvider
-from trac.cache import CacheManager
-from trac.config import *
+from trac.cache import CacheManager, cached
+from trac.config import BoolOption, ConfigSection, Configuration, Option, \
+                        PathOption
 from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracError
 from trac.db.api import (DatabaseManager, QueryContextManager,
@@ -38,7 +39,6 @@ from trac.util.concurrency import threading
 from trac.util.text import exception_to_unicode, path_to_unicode, printerr, \
                            printout
 from trac.util.translation import _, N_
-from trac.versioncontrol import RepositoryManager
 from trac.web.href import Href
 
 __all__ = ['Environment', 'IEnvironmentSetupParticipant', 'open_environment']
@@ -275,7 +275,6 @@ class Environment(Component, ComponentManager):
 
         self.path = path
         self.systeminfo = []
-        self._href = self._abs_href = None
 
         if create:
             self.create(options)
@@ -287,6 +286,14 @@ class Environment(Component, ComponentManager):
             for setup_participant in self.setup_participants:
                 setup_participant.environment_created()
 
+    @property
+    def env(self):
+        """Property returning the `Environment` object, which is often
+        required for functions and methods that take a `Component` instance.
+        """
+        # The cached decorator requires the object have an `env` attribute.
+        return self
+
     def get_systeminfo(self):
         """Return a list of `(name, version)` tuples describing the
         name and version information of external packages used by Trac
@@ -295,8 +302,8 @@ class Environment(Component, ComponentManager):
         info = self.systeminfo[:]
         for provider in self.system_info_providers:
             info.extend(provider.get_system_info() or [])
-        info.sort(key=lambda (name, version): (name != 'Trac', name.lower()))
-        return info
+        return sorted(set(info),
+                      key=lambda (name, ver): (name != 'Trac', name.lower()))
 
     # ISystemInfoProvider methods
 
@@ -326,17 +333,14 @@ class Environment(Component, ComponentManager):
             name = name_or_class.__module__ + '.' + name_or_class.__name__
         return name.lower()
 
-    @property
+    @lazy
     def _component_rules(self):
-        try:
-            return self._rules
-        except AttributeError:
-            self._rules = {}
-            for name, value in self.components_section.options():
-                if name.endswith('.*'):
-                    name = name[:-2]
-                self._rules[name.lower()] = value.lower() in ('enabled', 'on')
-            return self._rules
+        _rules = {}
+        for name, value in self.components_section.options():
+            if name.endswith('.*'):
+                name = name[:-2]
+            _rules[name.lower()] = value.lower() in ('enabled', 'on')
+        return _rules
 
     def is_component_enabled(self, cls):
         """Implemented to only allow activation of components that are
@@ -373,12 +377,15 @@ class Environment(Component, ComponentManager):
                 break
             cname = cname[:idx]
 
-        # By default, all components in the trac package are enabled
-        return component_name.startswith('trac.') or None
+        # By default, all components in the trac package except
+        # trac.test are enabled
+        return component_name.startswith('trac.') and \
+               not component_name.startswith('trac.test.') or None
 
     def enable_component(self, cls):
         """Enable a component or module."""
         self._component_rules[self._component_name(cls)] = True
+        super(Environment, self).enable_component(cls)
 
     def verify(self):
         """Verify that the provided path points to a valid Trac environment
@@ -386,15 +393,17 @@ class Environment(Component, ComponentManager):
         try:
             tag = read_file(os.path.join(self.path, 'VERSION')).splitlines()[0]
             if tag != _VERSION:
-                raise Exception("Unknown Trac environment type '%s'" % tag)
+                raise Exception(_("Unknown Trac environment type '%(type)s'",
+                                  type=tag))
         except Exception, e:
-            raise TracError("No Trac environment found at %s\n%s"
-                            % (self.path, e))
+            raise TracError(_("No Trac environment found at %(path)s\n"
+                              "%(e)s", path=self.path, e=e))
 
     def get_db_cnx(self):
         """Return a database connection from the connection pool
 
-        :deprecated: Use :meth:`db_transaction` or :meth:`db_query` instead
+        :deprecated: Use :meth:`db_transaction` or :meth:`db_query` instead.
+                     Removed in Trac 1.1.2.
 
         `db_transaction` for obtaining the `db` database connection
         which can be used for performing any query
@@ -435,13 +444,21 @@ class Environment(Component, ComponentManager):
         return DatabaseManager(self).get_exceptions()
 
     def with_transaction(self, db=None):
-        """Decorator for transaction functions :deprecated:"""
+        """Decorator for transaction functions.
+
+        :deprecated: Use the query and transaction context managers instead.
+                     Will be removed in Trac 1.3.1.
+        """
         return with_transaction(self, db)
 
     def get_read_db(self):
-        """Return a database connection for read purposes :deprecated:
+        """Return a database connection for read purposes.
 
-        See `trac.db.api.get_read_db` for detailed documentation."""
+        See `trac.db.api.get_read_db` for detailed documentation.
+
+        :deprecated: Use :meth:`db_query` instead.
+                     Will be removed in Trac 1.3.1.
+        """
         return DatabaseManager(self).get_connection(readonly=True)
 
     @property
@@ -522,6 +539,7 @@ class Environment(Component, ComponentManager):
 
     def shutdown(self, tid=None):
         """Close the environment."""
+        from trac.versioncontrol.api import RepositoryManager
         RepositoryManager(self).shutdown(tid)
         DatabaseManager(self).shutdown(tid)
         if tid is None:
@@ -543,6 +561,7 @@ class Environment(Component, ComponentManager):
                          anymore, left here for compatibility with
                          0.11)
         """
+        from trac.versioncontrol.api import RepositoryManager
         return RepositoryManager(self).get_repository(reponame)
 
     def create(self, options=[]):
@@ -557,9 +576,9 @@ class Environment(Component, ComponentManager):
         # Create the directory structure
         if not os.path.exists(self.path):
             os.mkdir(self.path)
-        os.mkdir(self.get_log_dir())
-        os.mkdir(self.get_htdocs_dir())
-        os.mkdir(os.path.join(self.path, 'plugins'))
+        os.mkdir(self.log_dir)
+        os.mkdir(self.htdocs_dir)
+        os.mkdir(self.plugins_dir)
 
         # Create a few files
         create_file(os.path.join(self.path, 'VERSION'), _VERSION + '\n')
@@ -568,9 +587,9 @@ class Environment(Component, ComponentManager):
                     'Visit http://trac.edgewall.org/ for more information.\n')
 
         # Setup the default configuration
-        os.mkdir(os.path.join(self.path, 'conf'))
-        create_file(os.path.join(self.path, 'conf', 'trac.ini.sample'))
-        config = Configuration(os.path.join(self.path, 'conf', 'trac.ini'))
+        os.mkdir(self.conf_dir)
+        create_file(os.path.join(self.conf_dir, 'trac.ini.sample'))
+        config = Configuration(os.path.join(self.conf_dir, 'trac.ini'))
         for section, name, value in options:
             config.set(section, name, value)
         config.save()
@@ -583,6 +602,25 @@ class Environment(Component, ComponentManager):
         # Create the database
         DatabaseManager(self).init_db()
 
+    @lazy
+    def database_version(self):
+        """Returns the current version of the database.
+
+        :since 1.0.2:
+        """
+        return self.get_version()
+
+    @lazy
+    def database_initial_version(self):
+        """Returns the version of the database at the time of creation.
+
+        In practice, for database created before 0.11, this will
+        return `False` which is "older" than any db version number.
+
+        :since 1.0.2:
+        """
+        return self.get_version(initial=True)
+
     def get_version(self, db=None, initial=False):
         """Return the current version of the database.  If the
         optional argument `initial` is set to `True`, the version of
@@ -595,32 +633,97 @@ class Environment(Component, ComponentManager):
 
         :since 1.0: deprecation warning: the `db` parameter is no
                     longer used and will be removed in version 1.1.1
+
+        :since 1.0.2: The lazily-evaluated attributes `database_version` and
+                      `database_initial_version` should be used instead. This
+                      method will be renamed to a private method in
+                      release 1.3.1.
         """
         rows = self.db_query("""
                 SELECT value FROM system WHERE name='%sdatabase_version'
                 """ % ('initial_' if initial else ''))
-        return rows and int(rows[0][0])
+        return int(rows[0][0]) if rows else False
 
     def setup_config(self):
         """Load the configuration file."""
-        self.config = Configuration(os.path.join(self.path, 'conf', 'trac.ini'),
+        config_file_path = os.path.join(self.conf_dir, 'trac.ini')
+        self.config = Configuration(config_file_path,
                                     {'envname': os.path.basename(self.path)})
+        if not self.config.exists:
+            raise TracError(_("The configuration file is not found at "
+                              "%(path)s", path=config_file_path))
         self.setup_log()
         from trac.loader import load_components
         plugins_dir = self.shared_plugins_dir
         load_components(self, plugins_dir and (plugins_dir,))
 
-    def get_templates_dir(self):
-        """Return absolute path to the templates directory."""
-        return os.path.join(self.path, 'templates')
+    def _get_path_to_dir(self, dir):
+        path = os.path.join(self.path, dir)
+        return os.path.normcase(os.path.realpath(path))
+
+    @lazy
+    def conf_dir(self):
+        """Absolute path to the conf directory.
+
+        :since: 1.0.11
+        """
+        return self._get_path_to_dir('conf')
+
+    @lazy
+    def htdocs_dir(self):
+        """Absolute path to the htdocs directory.
+
+        :since: 1.0.11
+        """
+        return self._get_path_to_dir('htdocs')
 
     def get_htdocs_dir(self):
-        """Return absolute path to the htdocs directory."""
-        return os.path.join(self.path, 'htdocs')
+        """Return absolute path to the htdocs directory.
+
+        :since 1.0.11: Deprecated and will be removed in 1.3.1. Use the
+                       `htdocs_dir` property instead.
+        """
+        return self._get_path_to_dir('htdocs')
+
+    @lazy
+    def log_dir(self):
+        """Absolute path to the log directory.
+
+        :since: 1.0.11
+        """
+        return self._get_path_to_dir('log')
 
     def get_log_dir(self):
-        """Return absolute path to the log directory."""
-        return os.path.join(self.path, 'log')
+        """Return absolute path to the log directory.
+
+        :since 1.0.11: Deprecated and will be removed in 1.3.1. Use the
+                       `log_dir` property instead.
+        """
+        return self._get_path_to_dir('log')
+
+    @lazy
+    def plugins_dir(self):
+        """Absolute path to the plugins directory.
+
+        :since: 1.0.11
+        """
+        return self._get_path_to_dir('plugins')
+
+    @lazy
+    def templates_dir(self):
+        """Absolute path to the templates directory.
+
+        :since: 1.0.11
+        """
+        return self._get_path_to_dir('templates')
+
+    def get_templates_dir(self):
+        """Return absolute path to the templates directory.
+
+        :since 1.0.11: Deprecated and will be removed in 1.3.1. Use the
+                       `templates_dir` property instead.
+        """
+        return self._get_path_to_dir('templates')
 
     def setup_log(self):
         """Initialize the logging sub-system."""
@@ -628,7 +731,7 @@ class Environment(Component, ComponentManager):
         logtype = self.log_type
         logfile = self.log_file
         if logtype == 'file' and not os.path.isabs(logfile):
-            logfile = os.path.join(self.get_log_dir(), logfile)
+            logfile = os.path.join(self.log_dir, logfile)
         format = self.log_format
         logid = 'Trac.%s' % sha1(self.path).hexdigest()
         if format:
@@ -656,16 +759,23 @@ class Environment(Component, ComponentManager):
         :since 1.0: deprecation warning: the `cnx` parameter is no
                     longer used and will be removed in version 1.1.1
         """
-        for username, name, email in self.db_query("""
+        return iter(self._known_users)
+
+    @cached
+    def _known_users(self):
+        return self.db_query("""
                 SELECT DISTINCT s.sid, n.value, e.value
                 FROM session AS s
                  LEFT JOIN session_attribute AS n ON (n.sid=s.sid
-                  and n.authenticated=1 AND n.name = 'name')
+                  AND n.authenticated=1 AND n.name = 'name')
                  LEFT JOIN session_attribute AS e ON (e.sid=s.sid
                   AND e.authenticated=1 AND e.name = 'email')
                 WHERE s.authenticated=1 ORDER BY s.sid
-                """):
-            yield username, name, email
+        """)
+
+    def invalidate_known_users_cache(self):
+        """Clear the known_users cache."""
+        del self._known_users
 
     def backup(self, dest=None):
         """Create a backup of the database.
@@ -713,26 +823,21 @@ class Environment(Component, ComponentManager):
                 participant.upgrade_environment(db)
             # Database schema may have changed, so close all connections
             DatabaseManager(self).shutdown()
+        del self.database_version
         return True
 
-    @property
+    @lazy
     def href(self):
         """The application root path"""
-        if not self._href:
-            self._href = Href(urlsplit(self.abs_href.base)[2])
-        return self._href
+        return Href(urlsplit(self.abs_href.base).path)
 
-    @property
+    @lazy
     def abs_href(self):
         """The application URL"""
-        if not self._abs_href:
-            if not self.base_url:
-                self.log.warn("base_url option not set in configuration, "
-                              "generated links may be incorrect")
-                self._abs_href = Href('')
-            else:
-                self._abs_href = Href(self.base_url)
-        return self._abs_href
+        if not self.base_url:
+            self.log.warn("base_url option not set in configuration, "
+                          "generated links may be incorrect")
+        return Href(self.base_url)
 
 
 class EnvironmentSetup(Component):
@@ -754,7 +859,7 @@ class EnvironmentSetup(Component):
         self._update_sample_config()
 
     def environment_needs_upgrade(self, db):
-        dbver = self.env.get_version(db)
+        dbver = self.env.database_version
         if dbver == db_default.db_version:
             return False
         elif dbver > db_default.db_version:
@@ -768,7 +873,7 @@ class EnvironmentSetup(Component):
         upgrades/dbN.py, where 'N' is the version number (int).
         """
         cursor = db.cursor()
-        dbver = self.env.get_version()
+        dbver = self.env.database_version
         for i in range(dbver + 1, db_default.db_version + 1):
             name  = 'db%i' % i
             try:
@@ -788,13 +893,12 @@ class EnvironmentSetup(Component):
     # Internal methods
 
     def _update_sample_config(self):
-        filename = os.path.join(self.env.path, 'conf', 'trac.ini.sample')
+        filename = os.path.join(self.env.conf_dir, 'trac.ini.sample')
         if not os.path.isfile(filename):
             return
         config = Configuration(filename)
-        for section, default_options in config.defaults().iteritems():
-            for name, value in default_options.iteritems():
-                config.set(section, name, value)
+        for (section, name), option in Option.get_registry().iteritems():
+            config.set(section, name, option.dumps(option.default))
         try:
             config.save()
             self.log.info("Wrote sample configuration file with the new "
@@ -942,7 +1046,8 @@ class EnvironmentAdmin(Component):
             if prefix == 'sqlite':
                 db_path = os.path.join(self.env.path, os.path.normpath(db_path))
                 # don't copy the journal (also, this would fail on Windows)
-                skip = [db_path + '-journal', db_path + '-stmtjrnl']
+                skip = [db_path + '-journal', db_path + '-stmtjrnl',
+                        db_path + '-shm', db_path + '-wal']
                 if no_db:
                     skip.append(db_path)
 
