@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2006-2009 Edgewall Software
+# Copyright (C) 2006-2013 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,17 +14,19 @@
 import doctest
 import unittest
 from StringIO import StringIO
-import sys
 
-from trac.core import *
-from trac.test import EnvironmentStub
-from trac.mimeview import api
-from trac.mimeview.api import get_mimetype, IContentConverter, Mimeview, \
-                              _group_lines
 from genshi import Stream, Namespace
 from genshi.core import Attrs, TEXT, START, END
 from genshi.input import HTMLParser
 
+import trac.tests.compat
+from trac.core import Component, implements
+from trac.test import EnvironmentStub, MockRequest
+from trac.mimeview import api
+from trac.mimeview.api import get_mimetype, IContentConverter, Mimeview, \
+                              _group_lines
+from trac.web.api import RequestDone
+
 
 class GetMimeTypeTestCase(unittest.TestCase):
 
@@ -112,22 +114,22 @@ class GroupLinesTestCase(unittest.TestCase):
     def test_text_only_stream(self):
         input = [(TEXT, "test", (None, -1, -1))]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), 1)
-        self.assertTrue(isinstance(lines[0], Stream))
-        self.assertEquals(lines[0].events, input)
+        self.assertEqual(len(lines), 1)
+        self.assertIsInstance(lines[0], Stream)
+        self.assertEqual(lines[0].events, input)
 
     def test_text_only_stream2(self):
         input = [(TEXT, "test\n", (None, -1, -1))]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), 1)
-        self.assertTrue(isinstance(lines[0], Stream))
-        self.assertEquals(lines[0].events, [(TEXT, "test", (None, -1, -1))])
+        self.assertEqual(len(lines), 1)
+        self.assertIsInstance(lines[0], Stream)
+        self.assertEqual(lines[0].events, [(TEXT, "test", (None, -1, -1))])
 
     def test_simplespan(self):
         input = HTMLParser(StringIO(u"<span>test</span>"), encoding=None)
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), 1)
-        self.assertTrue(isinstance(lines[0], Stream))
+        self.assertEqual(len(lines), 1)
+        self.assertIsInstance(lines[0], Stream)
         for (a, b) in zip(lines[0], input):
             self.assertEqual(a, b)
 
@@ -137,17 +139,17 @@ class GroupLinesTestCase(unittest.TestCase):
         """
         input = [(TEXT, "", (None, -1, -1))]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), 0)
+        self.assertEqual(len(lines), 0)
 
     def test_newline_stream(self):
         input = [(TEXT, "\n", (None, -1, -1))]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), 1)
+        self.assertEqual(len(lines), 1)
 
     def test_newline_stream2(self):
         input = [(TEXT, "\n\n\n", (None, -1, -1))]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), 3)
+        self.assertEqual(len(lines), 3)
 
     def test_empty_text_in_span(self):
         """
@@ -172,9 +174,9 @@ class GroupLinesTestCase(unittest.TestCase):
                     '<span class="c">b</span>',
                    ]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), len(expected))
+        self.assertEqual(len(lines), len(expected))
         for a, b in zip(lines, expected):
-            self.assertEquals(a.render('html'), b)
+            self.assertEqual(a.render('html'), b)
 
     def test_newline2(self):
         """
@@ -187,9 +189,9 @@ class GroupLinesTestCase(unittest.TestCase):
                     '<span class="c">b</span>',
                    ]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), len(expected))
+        self.assertEqual(len(lines), len(expected))
         for a, b in zip(lines, expected):
-            self.assertEquals(a.render('html'), b)
+            self.assertEqual(a.render('html'), b)
 
     def test_multinewline(self):
         """
@@ -203,17 +205,136 @@ class GroupLinesTestCase(unittest.TestCase):
                     '<span class="c">a</span>',
                    ]
         lines = list(_group_lines(input))
-        self.assertEquals(len(lines), len(expected))
+        self.assertEqual(len(lines), len(expected))
         for a, b in zip(lines, expected):
-            self.assertEquals(a.render('html'), b)
+            self.assertEqual(a.render('html'), b)
+
+
+class TestMimeviewConverter(Component):
+
+    implements(IContentConverter)
+
+    in_mimetype = __module__ + '.' + __name__
+
+    def get_supported_conversions(self):
+        yield ('text', self.__module__, 'txt', self.in_mimetype, 'text/plain',
+               8)
+
+    def convert_content(self, req, mimetype, content, key):
+        if content == 'iterable-bytes':
+            def fn_bytes():
+                for idx in xrange(256):
+                    yield 'b' * 256
+            return fn_bytes(), 'text/plain'
+        if content == 'iterable-unicode':
+            def fn_unicode():
+                for idx in xrange(0x10000):
+                    yield u'ü'
+            return fn_unicode(), 'text/plain'
+        if content == 'bytes':
+            return 'a' * 0x10000, 'text/plain'
+        if content == 'unicode':
+            return u'Ü' * 0x10000, 'text/plain'
+
+
+class MimeviewConverterTestCase(unittest.TestCase):
+
+    in_mimetype = TestMimeviewConverter.in_mimetype
+
+    def setUp(self):
+        self.env = EnvironmentStub(enable=['trac.*', TestMimeviewConverter])
+
+    def tearDown(self):
+        pass
+
+    def _test_convert_content(self, expected, content, iterable):
+        mimeview = Mimeview(self.env)
+        output = mimeview.convert_content(MockRequest(self.env),
+                                          self.in_mimetype,
+                                          content, 'text', iterable=iterable)
+        if iterable:
+            self.assertNotIn(type(output[0]), (str, unicode))
+            self.assertEqual(expected, ''.join(output[0]))
+        else:
+            self.assertEqual(type(expected), type(output[0]))
+            self.assertEqual(expected, output[0])
+        self.assertEqual('text/plain', output[1])
+        self.assertEqual('txt', output[2])
+
+    def test_convert_content_iterable_bytes(self):
+        self._test_convert_content('b' * 0x10000, 'iterable-bytes', False)
+
+    def test_convert_content_iterable_unicode(self):
+        self._test_convert_content(u'ü' * 0x10000, 'iterable-unicode', False)
+
+    def test_convert_content_bytes(self):
+        self._test_convert_content('a' * 0x10000, 'bytes', False)
+
+    def test_convert_content_unicode(self):
+        self._test_convert_content(u'Ü' * 0x10000, 'unicode', False)
+
+    def test_convert_content_iterable_bytes_iterable(self):
+        self._test_convert_content('b' * 0x10000, 'iterable-bytes', True)
+
+    def test_convert_content_iterable_unicode_iterable(self):
+        self._test_convert_content(u'ü' * 0x10000, 'iterable-unicode', True)
+
+    def test_convert_content_bytes_iterable(self):
+        self._test_convert_content('a' * 0x10000, 'bytes', True)
+
+    def test_convert_content_unicode_iterable(self):
+        self._test_convert_content(u'Ü' * 0x10000, 'unicode', True)
+
+    def _test_send_converted(self, expected, content, use_chunked_encoding):
+        self.env.config.set('trac', 'use_chunked_encoding',
+                            'true' if use_chunked_encoding else 'false')
+        mimeview = Mimeview(self.env)
+        req = MockRequest(self.env)
+        self.assertRaises(RequestDone, mimeview.send_converted, req,
+                          self.in_mimetype, content, 'text')
+        result = req.response_sent.getvalue()
+        if use_chunked_encoding:
+            self.assertNotIn('Content-Length', req.headers_sent)
+        else:
+            self.assertIn('Content-Length', req.headers_sent)
+            self.assertEqual(str(len(expected)),
+                             req.headers_sent['Content-Length'])
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(set(expected), set(result))
+        self.assertEqual(expected, result)
+
+    def test_send_converted_iterable_bytes(self):
+        self._test_send_converted('b' * 0x10000, 'iterable-bytes', False)
+
+    def test_send_converted_iterable_unicode(self):
+        self._test_send_converted('ü' * 0x10000, 'iterable-unicode', False)
+
+    def test_send_converted_bytes(self):
+        self._test_send_converted('a' * 0x10000, 'bytes', False)
+
+    def test_send_converted_unicode(self):
+        self._test_send_converted('Ü' * 0x10000, 'unicode', False)
+
+    def test_send_converted_iterable_bytes_chunked(self):
+        self._test_send_converted('b' * 0x10000, 'iterable-bytes', True)
+
+    def test_send_converted_iterable_unicode_chunked(self):
+        self._test_send_converted('ü' * 0x10000, 'iterable-unicode', True)
+
+    def test_send_converted_bytes_chunked(self):
+        self._test_send_converted('a' * 0x10000, 'bytes', True)
+
+    def test_send_converted_unicode_chunked(self):
+        self._test_send_converted('Ü' * 0x10000, 'unicode', True)
 
 
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(api))
-    suite.addTest(unittest.makeSuite(GetMimeTypeTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(MimeviewTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(GroupLinesTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(GetMimeTypeTestCase))
+    suite.addTest(unittest.makeSuite(MimeviewTestCase))
+    suite.addTest(unittest.makeSuite(GroupLinesTestCase))
+    suite.addTest(unittest.makeSuite(MimeviewConverterTestCase))
     return suite
 
 if __name__ == '__main__':
