@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2006-2009 Edgewall Software
+# Copyright (C) 2006-2013 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -17,7 +17,7 @@ import unittest
 from genshi.core import Stream
 from genshi.input import HTMLParser, XML
 
-from trac.mimeview.api import Mimeview, RenderingContext
+from trac.mimeview.api import Mimeview
 from trac.mimeview.patch import PatchRenderer
 from trac.test import EnvironmentStub, Mock, MockPerm
 from trac.web.chrome import Chrome, web_context
@@ -45,8 +45,8 @@ class PatchRendererTestCase(unittest.TestCase):
         result = XML(result.render(encoding='utf-8')).render(encoding='utf-8')
         expected, result = expected.splitlines(), result.splitlines()
         for exp, res in zip(expected, result):
-            self.assertEquals(exp, res)
-        self.assertEquals(len(expected), len(result))
+            self.assertEqual(exp, res)
+        self.assertEqual(len(expected), len(result))
 
     def test_simple(self):
         """
@@ -105,10 +105,10 @@ class PatchRendererTestCase(unittest.TestCase):
              '@@ -1 +1 @@',
              '-aa\tb',
              '+aaxb'], 8)
-        self.assertEquals('aa<del>&nbsp; &nbsp; &nbsp; </del>b',
-                          str(changes[0]['diffs'][0][0]['base']['lines'][0]))
-        self.assertEquals('aa<ins>x</ins>b',
-                          str(changes[0]['diffs'][0][0]['changed']['lines'][0]))
+        self.assertEqual('aa<del>&nbsp; &nbsp; &nbsp; </del>b',
+                         str(changes[0]['diffs'][0][0]['base']['lines'][0]))
+        self.assertEqual('aa<ins>x</ins>b',
+                         str(changes[0]['diffs'][0][0]['changed']['lines'][0]))
 
     def test_diff_to_hdf_leading_ws(self):
         """Regression test related to #5795"""
@@ -118,14 +118,26 @@ class PatchRendererTestCase(unittest.TestCase):
              '@@ -1 +1 @@',
              '-*a',
              '+ *a'], 8)
-        self.assertEquals('<del></del>*a',
-                          str(changes[0]['diffs'][0][0]['base']['lines'][0]))
-        self.assertEquals('<ins>&nbsp;</ins>*a',
-                          str(changes[0]['diffs'][0][0]['changed']['lines'][0]))
+        self.assertEqual('<del></del>*a',
+                         str(changes[0]['diffs'][0][0]['base']['lines'][0]))
+        self.assertEqual('<ins>&nbsp;</ins>*a',
+                         str(changes[0]['diffs'][0][0]['changed']['lines'][0]))
+
+    def test_range_information_with_no_lines(self):
+        result = self.patch.render(self.context, None, """
+Index: filename.txt
+===================================================================
+--- filename.txt
++++ filename.txt
+@@ -14,7 +14,7 @@
+""")
+        self.assertTrue(result)
+        self._test('range_information_with_no_lines', result)
+
 
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(PatchRendererTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(PatchRendererTestCase))
     return suite
 
 if __name__ == '__main__':
