@@ -17,16 +17,18 @@ import os
 import re
 import smtplib
 from subprocess import Popen, PIPE
-import time
 
 from genshi.builder import tag
 
 from trac import __version__
-from trac.config import BoolOption, ExtensionOption, IntOption, Option
+from trac.config import BoolOption, ConfigurationError, ExtensionOption, \
+                        IntOption, Option
 from trac.core import *
 from trac.util.compat import close_fds
-from trac.util.text import CRLF, fix_eol
-from trac.util.translation import _, deactivate, reactivate
+from trac.util.datefmt import time_now
+from trac.util.html import to_fragment
+from trac.util.text import CRLF, fix_eol, to_unicode
+from trac.util.translation import _, deactivate, reactivate, tag_
 
 MAXHEADERLEN = 76
 EMAIL_LOOKALIKE_PATTERN = (
@@ -37,6 +39,8 @@ EMAIL_LOOKALIKE_PATTERN = (
         '[a-zA-Z](?:[-a-zA-Z\d]*[a-zA-Z\d])?' # TLD
         )
 
+local_hostname = None
+
 
 class IEmailSender(Interface):
     """Extension point interface for components that allow sending e-mail."""
@@ -120,6 +124,10 @@ class NotificationSystem(Component):
         If no prefix is desired, then specifying an empty option
         will disable it. (''since 0.10.1'')""")
 
+    message_id_hash = Option('notification', 'message_id_hash', 'md5',
+        """Hash algorithm to create unique Message-ID header.
+        ''(since 1.0.13)''""")
+
     def send_email(self, from_addr, recipients, message):
         """Send message to recipients via e-mail."""
         self.email_sender.send(from_addr, recipients, message)
@@ -146,29 +154,41 @@ class SmtpEmailSender(Component):
         """Use SSL/TLS to send notifications over SMTP. (''since 0.10'')""")
 
     def send(self, from_addr, recipients, message):
+        global local_hostname
         # Ensure the message complies with RFC2822: use CRLF line endings
         message = fix_eol(message, CRLF)
 
-        self.log.info("Sending notification through SMTP at %s:%d to %s"
-                      % (self.smtp_server, self.smtp_port, recipients))
-        server = smtplib.SMTP(self.smtp_server, self.smtp_port)
+        self.log.info("Sending notification through SMTP at %s:%d to %s",
+                      self.smtp_server, self.smtp_port, recipients)
+        try:
+            server = smtplib.SMTP(self.smtp_server, self.smtp_port,
+                                  local_hostname)
+            local_hostname = server.local_hostname
+        except smtplib.socket.error, e:
+            raise ConfigurationError(
+                tag_("SMTP server connection error (%(error)s). Please "
+                     "modify %(option1)s or %(option2)s in your "
+                     "configuration.",
+                     error=to_unicode(e),
+                     option1=tag.tt("[notification] smtp_server"),
+                     option2=tag.tt("[notification] smtp_port")))
         # server.set_debuglevel(True)
         if self.use_tls:
             server.ehlo()
-            if not server.esmtp_features.has_key('starttls'):
-                raise TracError(_("TLS enabled but server does not support " \
+            if 'starttls' not in server.esmtp_features:
+                raise TracError(_("TLS enabled but server does not support "
                                   "TLS"))
             server.starttls()
             server.ehlo()
         if self.smtp_user:
             server.login(self.smtp_user.encode('utf-8'),
                          self.smtp_password.encode('utf-8'))
-        start = time.time()
+        start = time_now()
         server.sendmail(from_addr, recipients, message)
-        t = time.time() - start
+        t = time_now() - start
         if t > 5:
             self.log.warning('Slow mail submission (%.2f s), '
-                             'check your mail setup' % t)
+                             'check your mail setup', t)
         if self.use_tls:
             # avoid false failure detection when the server closes
             # the SMTP connection with TLS enabled
@@ -190,19 +210,27 @@ class SendmailEmailSender(Component):
         """Path to the sendmail executable.
 
         The sendmail program must accept the `-i` and `-f` options.
-         (''since 0.12'')""")
+        (''since 0.12'')
+        """)
 
     def send(self, from_addr, recipients, message):
         # Use native line endings in message
         message = fix_eol(message, os.linesep)
 
-        self.log.info("Sending notification through sendmail at %s to %s"
-                      % (self.sendmail_path, recipients))
+        self.log.info("Sending notification through sendmail at %s to %s",
+                      self.sendmail_path, recipients)
         cmdline = [self.sendmail_path, "-i", "-f", from_addr]
         cmdline.extend(recipients)
-        self.log.debug("Sendmail command line: %s" % cmdline)
-        child = Popen(cmdline, bufsize=-1, stdin=PIPE, stdout=PIPE,
-                      stderr=PIPE, close_fds=close_fds)
+        self.log.debug("Sendmail command line: %s", cmdline)
+        try:
+            child = Popen(cmdline, bufsize=-1, stdin=PIPE, stdout=PIPE,
+                          stderr=PIPE, close_fds=close_fds)
+        except OSError, e:
+            raise ConfigurationError(
+                tag_("Sendmail error (%(error)s). Please modify %(option)s "
+                     "in your configuration.",
+                     error=to_unicode(e),
+                     option=tag.tt("[notification] sendmail_path")))
         out, err = child.communicate(message)
         if child.returncode or err:
             raise Exception("Sendmail failed with (%s, %s), command: '%s'"
@@ -227,7 +255,7 @@ class Notify(object):
         self.data = Chrome(self.env).populate_data(None, {'CRLF': CRLF})
 
     def notify(self, resid):
-        (torcpts, ccrcpts) = self.get_recipients(resid)
+        torcpts, ccrcpts = self.get_recipients(resid)
         self.begin_send()
         self.send(torcpts, ccrcpts)
         self.finish_send()
@@ -333,30 +361,44 @@ class NotifyEmail(Notify):
         self.from_email = from_email or self.replyto_email
         self.from_name = from_name
         if not self.from_email and not self.replyto_email:
-            raise TracError(tag(
-                    tag.p(_('Unable to send email due to identity crisis.')),
-                    tag.p(_('Neither %(from_)s nor %(reply_to)s are specified '
-                            'in the configuration.',
-                            from_=tag.b('notification.from'),
-                            reply_to=tag.b('notification.reply_to')))),
-                _('SMTP Notification Error'))
+            message = tag(
+                tag.p(_('Unable to send email due to identity crisis.')),
+                # convert explicitly to `Fragment` to avoid breaking message
+                # when passing `LazyProxy` object to `Fragment`
+                tag.p(to_fragment(tag_(
+                    "Neither %(from_)s nor %(reply_to)s are specified in the "
+                    "configuration.",
+                    from_=tag.strong('[notification] smtp_from'),
+                    reply_to=tag.strong('[notification] smtp_replyto')))))
+            raise TracError(message, _('SMTP Notification Error'))
 
         Notify.notify(self, resid)
 
+    _mime_encoding_re = re.compile(r'=\?[^?]+\?[bq]\?[^?]+\?=', re.IGNORECASE)
+
     def format_header(self, key, name, email=None):
         from email.Header import Header
         maxlength = MAXHEADERLEN-(len(key)+2)
         # Do not sent ridiculous short headers
         if maxlength < 10:
             raise TracError(_("Header length is too short"))
-        try:
-            tmp = name.encode('ascii')
-            header = Header(tmp, 'ascii', maxlinelen=maxlength)
-        except UnicodeEncodeError:
-            header = Header(name, self._charset, maxlinelen=maxlength)
+        # when it matches mime-encoding, encode as mime even if only
+        # ascii characters
+        header = None
+        if not self._mime_encoding_re.search(name):
+            try:
+                tmp = name.encode('ascii')
+                header = Header(tmp, 'ascii', maxlinelen=maxlength)
+            except UnicodeEncodeError:
+                pass
+        if not header:
+            header = Header(name.encode(self._charset.output_codec),
+                            self._charset, maxlinelen=maxlength)
         if not email:
             return header
         else:
+            header = str(header).replace('\\', r'\\') \
+                                .replace('"', r'\"')
             return '"%s" <%s>' % (header, email)
 
     def add_headers(self, msg, headers):
@@ -386,7 +428,7 @@ class NotifyEmail(Notify):
             if domain:
                 address = "%s@%s" % (address, domain)
             else:
-                self.env.log.info("Email address w/o domain: %s" % address)
+                self.env.log.info("Email address w/o domain: %s", address)
                 return None
 
         mo = self.shortaddr_re.search(address)
@@ -395,7 +437,7 @@ class NotifyEmail(Notify):
         mo = self.longaddr_re.search(address)
         if mo:
             return mo.group(2)
-        self.env.log.info("Invalid email address: %s" % address)
+        self.env.log.info("Invalid email address: %s", address)
         return None
 
     def encode_header(self, key, value):
@@ -460,7 +502,7 @@ class NotifyEmail(Notify):
 
         # if there is not valid recipient, leave immediately
         if len(recipients) < 1:
-            self.env.log.info('no recipient for a ticket notification')
+            self.env.log.info("no recipient for a ticket notification")
             return
 
         pcc = accaddrs
