@@ -1,5 +1,18 @@
-import os
-import shutil
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2006-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
+
+from __future__ import with_statement
+
 import tempfile
 import unittest
 
@@ -11,6 +24,7 @@ from trac.test import MockPerm
 from trac.web.href import Href
 from trac.wiki.tests import formatter
 
+
 SEARCH_TEST_CASES = u"""
 ============================== search: link resolver
 search:foo
@@ -116,18 +130,19 @@ attachment:file.txt?format=raw
 def attachment_setup(tc):
     import trac.ticket.api
     import trac.wiki.api
-    tc.env.path = os.path.join(tempfile.gettempdir(), 'trac-tempenv')
-    os.mkdir(tc.env.path)
-    attachment = Attachment(tc.env, 'wiki', 'WikiStart')
-    attachment.insert('file.txt', tempfile.TemporaryFile(), 0)
+    tc.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
+    with tc.env.db_transaction as db:
+        db("INSERT INTO wiki (name,version) VALUES ('SomePage/SubPage',1)")
+        db("INSERT INTO ticket (id) VALUES (123)")
     attachment = Attachment(tc.env, 'ticket', 123)
     attachment.insert('file.txt', tempfile.TemporaryFile(), 0)
+    attachment = Attachment(tc.env, 'wiki', 'WikiStart')
+    attachment.insert('file.txt', tempfile.TemporaryFile(), 0)
     attachment = Attachment(tc.env, 'wiki', 'SomePage/SubPage')
     attachment.insert('foo.txt', tempfile.TemporaryFile(), 0)
 
 def attachment_teardown(tc):
-    shutil.rmtree(tc.env.path)
-    tc.env.reset_db()
+    tc.env.reset_db_and_disk()
 
 
 EMAIL_TEST_CASE_DEFAULT = u"""
@@ -188,4 +203,3 @@ def suite():
 
 if __name__ == '__main__':
     unittest.main(defaultTest='suite')
-
