@@ -17,7 +17,9 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Matthew Good <trac@matt-good.net>
 
+import inspect
 import math
+import os
 import re
 import sys
 import time
@@ -26,25 +28,105 @@ from locale import getlocale, LC_TIME
 
 try:
     import babel
+except ImportError:
+    babel = None
+else:
     from babel import Locale
-    from babel.core import LOCALE_ALIASES
+    from babel.core import LOCALE_ALIASES, UnknownLocaleError
     from babel.dates import (
         format_datetime as babel_format_datetime,
         format_date as babel_format_date,
         format_time as babel_format_time,
         get_datetime_format, get_date_format,
         get_time_format, get_month_names,
-        get_period_names, get_day_names
+        get_period_names as babel_get_period_names,
+        get_day_names
     )
-except ImportError:
-    babel = None
+    if 'context' in inspect.getargspec(babel_get_period_names)[0]:
+        def get_period_names(locale=None):
+            return babel_get_period_names(context='format', locale=locale)
+    else:
+        get_period_names = babel_get_period_names
 
 from trac.core import TracError
 from trac.util.text import to_unicode, getpreferredencoding
-from trac.util.translation import _, ngettext, get_available_locales
+from trac.util.translation import _, ngettext
 
 # Date/time utilities
 
+if os.name == 'nt':
+    def _precise_now_windows():
+        """Provide high-resolution system time if Windows 8+ and Windows
+        Server 2012+.
+        """
+        import ctypes
+        from ctypes.wintypes import DWORD, WORD
+
+        kernel32 = ctypes.windll.kernel32
+        GetLastError = kernel32.GetLastError
+        SystemTimeToFileTime = kernel32.SystemTimeToFileTime
+        try:
+            # GetSystemTimePreciseAsFileTime is available under Windows 8+
+            # and Windows Server 2012+
+            GetSystemTimePreciseAsFileTime = \
+                kernel32.GetSystemTimePreciseAsFileTime
+            get_systime = GetSystemTimePreciseAsFileTime
+            func_systime = 'GetSystemTimePreciseAsFileTime'
+        except AttributeError:
+            GetSystemTimePreciseAsFileTime = None
+            get_systime = kernel32.GetSystemTimeAsFileTime
+            func_systime = 'GetSystemTimeAsFileTime'
+
+        class FILETIME(ctypes.Structure):
+            _fields_ = [('dwLowDateTime', DWORD),
+                        ('dwHighDateTime', DWORD)]
+
+        class SYSTEMTIME(ctypes.Structure):
+            _fields_ = [('wYear', WORD),
+                        ('wMonth', WORD),
+                        ('wDayOfWeek', WORD),
+                        ('wDay', WORD),
+                        ('wHour', WORD),
+                        ('wMinute', WORD),
+                        ('wSecond', WORD),
+                        ('wMilliseconds', WORD)]
+
+        def get_filetime_epoch():
+            st = SYSTEMTIME()
+            st.wYear = 1970
+            st.wMonth = 1
+            st.wDay = 1
+            st.wDayOfWeek = 0
+            st.wHour = st.wMinute = st.wSecond = st.wMilliseconds = 0
+            ft = FILETIME()
+            if SystemTimeToFileTime(ctypes.pointer(st), ctypes.pointer(ft)):
+                return ft.dwHighDateTime * 0x100000000L + ft.dwLowDateTime
+            else:
+                raise RuntimeError('[LastError SystemTimeToFileTime %d]' %
+                                   GetLastError())
+
+        ft_epoch = get_filetime_epoch()
+
+        def time_now():
+            """Return the precise current time in seconds since the Epoch."""
+            ft = FILETIME()
+            if not get_systime(ctypes.pointer(ft)):
+                raise RuntimeError('[LastError %s %d]' %
+                                   (func_systime, GetLastError()))
+            ft = ft.dwHighDateTime * 0x100000000L + ft.dwLowDateTime
+            usec = (ft - ft_epoch) / 10L
+            return usec / 1000000.0
+
+        def datetime_now(tz=None):
+            """Return new datetime with precise current time."""
+            return datetime.fromtimestamp(time_now(), tz)
+
+        return time_now, datetime_now
+
+    time_now, datetime_now = _precise_now_windows()
+else:
+    time_now, datetime_now = time.time, datetime.now
+
 # -- conversion
 
 def to_datetime(t, tzinfo=None):
@@ -54,12 +136,14 @@ def to_datetime(t, tzinfo=None):
 
     ``t`` is converted using the following rules:
 
-     - If ``t`` is already a `datetime` object,
-       - if it is timezone-"naive", it is localized to ``tzinfo``
-       - if it is already timezone-aware, ``t`` is mapped to the given
-         timezone (`datetime.datetime.astimezone`)
-     - If ``t`` is None, the current time will be used.
-     - If ``t`` is a number, it is interpreted as a timestamp.
+    * If ``t`` is already a `datetime` object,
+
+     * if it is timezone-"naive", it is localized to ``tzinfo``
+     * if it is already timezone-aware, ``t`` is mapped to the given
+       timezone (`datetime.datetime.astimezone`)
+
+    * If ``t`` is None, the current time will be used.
+    * If ``t`` is a number, it is interpreted as a timestamp.
 
     Any other input will trigger a `TypeError`.
 
@@ -67,7 +151,7 @@ def to_datetime(t, tzinfo=None):
     """
     tz = tzinfo or localtz
     if t is None:
-        dt = datetime.now(tz)
+        dt = datetime_now(tz)
     elif isinstance(t, datetime):
         if t.tzinfo:
             dt = t.astimezone(tz)
@@ -87,6 +171,8 @@ def to_datetime(t, tzinfo=None):
                     timedelta(seconds=frac + 1)
         else:
             dt = datetime.fromtimestamp(t, tz)
+    else:
+        dt = None
     if dt:
         return tz.normalize(dt)
     raise TypeError('expecting datetime, int, long, float, or None; got %s' %
@@ -154,51 +240,45 @@ _BABEL_FORMATS = {
     'date': {'short': '%x', 'medium': '%x', 'long': '%x', 'full': '%x'},
     'time': {'short': '%H:%M', 'medium': '%X', 'long': '%X', 'full': '%X'},
 }
-_ISO8601_FORMATS = {
-    'datetime': {
-        '%x %X': 'iso8601', '%x': 'iso8601date', '%X': 'iso8601time',
-        'short': '%Y-%m-%dT%H:%M', 'medium': '%Y-%m-%dT%H:%M:%S',
-        'long': 'iso8601', 'full': 'iso8601',
-        'iso8601': 'iso8601', None: 'iso8601'},
-    'date': {
-        '%x %X': 'iso8601', '%x': 'iso8601date', '%X': 'iso8601time',
-        'short': 'iso8601date', 'medium': 'iso8601date',
-        'long': 'iso8601date', 'full': 'iso8601date',
-        'iso8601': 'iso8601date', None: 'iso8601date'},
-    'time': {
-        '%x %X': 'iso8601', '%x': 'iso8601date', '%X': 'iso8601time',
-        'short': '%H:%M', 'medium': '%H:%M:%S',
-        'long': 'iso8601time', 'full': 'iso8601time',
-        'iso8601': 'iso8601time', None: 'iso8601time'},
-}
 _STRFTIME_HINTS = {'%x %X': 'datetime', '%x': 'date', '%X': 'time'}
 
 def _format_datetime_without_babel(t, format):
-    normalize_Z = False
-    if format.lower().startswith('iso8601'):
-        if 'date' in format:
-            format = '%Y-%m-%d'
-        elif 'time' in format:
-            format = '%H:%M:%S%z'
-            normalize_Z = True
-        else:
-            format = '%Y-%m-%dT%H:%M:%S%z'
-            normalize_Z = True
     text = t.strftime(str(format))
-    if normalize_Z:
-        text = text.replace('+0000', 'Z')
-        if not text.endswith('Z'):
-            text = text[:-2] + ":" + text[-2:]
     encoding = getlocale(LC_TIME)[1] or getpreferredencoding() \
                or sys.getdefaultencoding()
     return unicode(text, encoding, 'replace')
 
+def _format_datetime_iso8601(t, format, hint):
+    if format != 'full':
+        t = t.replace(microsecond=0)
+    text = t.isoformat()  # YYYY-MM-DDThh:mm:ss.SSSSSS±hh:mm
+    if format == 'short':
+        text = text[:16]  # YYYY-MM-DDThh:mm
+    elif format == 'medium':
+        text = text[:19]  # YYYY-MM-DDThh:mm:ss
+    elif text.endswith('+00:00'):
+        text = text[:-6] + 'Z'
+    if hint == 'date':
+        text = text.split('T', 1)[0]
+    elif hint == 'time':
+        text = text.split('T', 1)[1]
+    return unicode(text, 'ascii')
+
 def _format_datetime(t, format, tzinfo, locale, hint):
     t = to_datetime(t, tzinfo or localtz)
 
-    if (format in ('iso8601', 'iso8601date', 'iso8601time') or
-        locale == 'iso8601'):
-        format = _ISO8601_FORMATS[hint].get(format, format)
+    if format == 'iso8601':
+        return _format_datetime_iso8601(t, 'long', hint)
+    if format in ('iso8601date', 'iso8601time'):
+        return _format_datetime_iso8601(t, 'long', format[7:])
+    if locale == 'iso8601':
+        if format is None:
+            format = 'long'
+        elif format in _STRFTIME_HINTS:
+            hint = _STRFTIME_HINTS[format]
+            format = 'long'
+        if format in ('short', 'medium', 'long', 'full'):
+            return _format_datetime_iso8601(t, format, hint)
         return _format_datetime_without_babel(t, format)
 
     if babel and locale:
@@ -254,11 +334,17 @@ def get_date_format_hint(locale=None):
     if babel and locale:
         format = get_date_format('medium', locale=locale)
         return format.pattern
+    return _libc_get_date_format_hint()
 
+def _libc_get_date_format_hint(format=None):
     t = datetime(1999, 10, 29, tzinfo=utc)
     tmpl = format_date(t, tzinfo=utc)
-    return tmpl.replace('1999', 'YYYY', 1).replace('99', 'YY', 1) \
-               .replace('10', 'MM', 1).replace('29', 'DD', 1)
+    units = [('1999', 'YYYY'), ('99', 'YY'), ('10', 'MM'), ('29', 'dd')]
+    if format:
+        units = [(unit[0], '%(' + unit[1] + ')s') for unit in units]
+    for unit in units:
+        tmpl = tmpl.replace(unit[0], unit[1], 1)
+    return tmpl
 
 def get_datetime_format_hint(locale=None):
     """Present the default format used by `format_datetime` in a human readable
@@ -274,16 +360,22 @@ def get_datetime_format_hint(locale=None):
         format = get_datetime_format('medium', locale=locale)
         return format.replace('{0}', time_pattern) \
                      .replace('{1}', date_pattern)
+    return _libc_get_datetime_format_hint()
 
+def _libc_get_datetime_format_hint(format=None):
     t = datetime(1999, 10, 29, 23, 59, 58, tzinfo=utc)
     tmpl = format_datetime(t, tzinfo=utc)
     ampm = format_time(t, '%p', tzinfo=utc)
+    units = []
     if ampm:
-        tmpl = tmpl.replace(ampm, 'a', 1)
-    return tmpl.replace('1999', 'YYYY', 1).replace('99', 'YY', 1) \
-               .replace('10', 'MM', 1).replace('29', 'DD', 1) \
-               .replace('23', 'hh', 1).replace('11', 'hh', 1) \
-               .replace('59', 'mm', 1).replace('58', 'ss', 1)
+        units.append((ampm, 'a'))
+    units.extend([('1999', 'YYYY'), ('99', 'YY'), ('10', 'MM'), ('29', 'dd'),
+                  ('23', 'hh'), ('11', 'hh'), ('59', 'mm'), ('58', 'ss')])
+    if format:
+        units = [(unit[0], '%(' + unit[1] + ')s') for unit in units]
+    for unit in units:
+        tmpl = tmpl.replace(unit[0], unit[1], 1)
+    return tmpl
 
 def get_month_names_jquery_ui(req):
     """Get the month names for the jQuery UI datepicker library"""
@@ -345,21 +437,25 @@ def get_time_format_jquery_ui(locale):
         return 'hh:mm:ssz'  # XXX timepicker doesn't support 'ISO_8601'
     if babel and locale:
         values = {'h': 'h', 'hh': 'hh', 'H': 'h', 'HH': 'hh',
-                  'm': 'm', 'mm': 'mm', 's': 's', 'ss': 'ss',
-                  'a': 'TT'}
-        return get_time_format('medium', locale=locale).format % values
+                  'm': 'm', 'mm': 'mm', 's': 's', 'ss': 'ss'}
+        f = get_time_format('medium', locale=locale).format
+        if '%(a)s' in f:
+            t = datetime(1999, 10, 29, 23, 59, 58, tzinfo=utc)
+            ampm = babel_format_datetime(t, 'a', None, locale)
+            values['a'] = 'TT' if ampm[0].isupper() else 'tt'
+        return f % values
 
     t = datetime(1999, 10, 29, 23, 59, 58, tzinfo=utc)
     tmpl = format_time(t, tzinfo=utc)
     ampm = format_time(t, '%p', tzinfo=utc)
     if ampm:
-        tmpl = tmpl.replace(ampm, 'TT', 1)
+        tmpl = tmpl.replace(ampm, 'TT' if ampm[0].isupper() else 'tt', 1)
     return tmpl.replace('23', 'hh', 1).replace('11', 'hh', 1) \
                .replace('59', 'mm', 1).replace('58', 'ss', 1)
 
 def get_timezone_list_jquery_ui(t=None):
     """Get timezone list for jQuery timepicker addon"""
-    t = datetime.now(utc) if t is None else utc.localize(t)
+    t = to_datetime(t, utc)
     zones = set(t.astimezone(get_timezone(tz)).strftime('%z')
                 for tz in all_timezones)
     return [{'value': 'Z', 'label': '+00:00'} \
@@ -372,11 +468,52 @@ def get_first_week_day_jquery_ui(req):
     if locale == 'iso8601':
         return 1 # Monday
     if babel and locale:
+        if not locale.territory:
+            # search first locale which has the same `langauge` and territory
+            # in preferred languages
+            for l in req.languages:
+                l = l.replace('-', '_').lower()
+                if l.startswith(locale.language.lower() + '_'):
+                    try:
+                        l = Locale.parse(l)
+                        if l.territory:
+                            locale = l
+                            break
+                    except (UnknownLocaleError, ValueError):
+                        pass
         if not locale.territory and locale.language in LOCALE_ALIASES:
             locale = Locale.parse(LOCALE_ALIASES[locale.language])
         return (locale.first_week_day + 1) % 7
     return 0 # Sunday
 
+def get_timepicker_separator_jquery_ui(req):
+    locale = req.lc_time
+    if locale == 'iso8601':
+        return 'T'
+    if babel and locale:
+        return get_datetime_format('medium', locale=locale) \
+               .replace('{0}', '').replace('{1}', '')
+    return ' '
+
+def get_period_names_jquery_ui(req):
+    # allow to use always English am/pm markers
+    english_names = {'am': 'AM', 'pm': 'PM'}
+    locale = req.lc_time
+    if locale == 'iso8601':
+        return {'am': [english_names['am']], 'pm': [english_names['pm']]}
+    if babel and locale:
+        names = get_period_names(locale=locale)
+        return dict((period, [names[period], english_names[period]])
+                    for period in ('am', 'pm'))
+    else:
+        # retrieve names of am/pm from libc
+        names = {}
+        for period, hour in (('am', 11), ('pm', 23)):
+            t = datetime(1999, 10, 29, hour, tzinfo=utc)
+            names[period] = [format_datetime(t, '%p', tzinfo=utc),
+                             english_names[period]]
+        return names
+
 def is_24_hours(locale):
     """Returns `True` for 24 hour time formats."""
     if locale == 'iso8601':
@@ -405,8 +542,11 @@ _ISO_8601_RE = re.compile(r'''
         (\d\d)(?::?(\d\d)(?::?(\d\d)        # time
         (?:[,.](\d{1,6}))?)?)?              # microseconds
     )?
-    (Z?(?:([-+])?(\d\d):?(\d\d)?)?)?$       # timezone
-    ''', re.VERBOSE)
+    (                                       # timezone
+        Z                                   #   Z
+      | ([-+])(\d\d):?(\d\d)?               #   ±hh:mm, ±hhmm, ±hh
+    )?
+    $''', re.VERBOSE)
 
 def _parse_date_iso8601(text, tzinfo):
     match = _ISO_8601_RE.match(text)
@@ -418,25 +558,44 @@ def _parse_date_iso8601(text, tzinfo):
             days = g[2] or '01'
             hours, minutes, seconds, useconds = [x or '00' for x in g[3:7]]
             useconds = (useconds + '000000')[:6]
-            z, tzsign, tzhours, tzminutes = g[7:11]
+            z = g[7]
             if z:
-                tz = timedelta(hours=int(tzhours or '0'),
-                               minutes=int(tzminutes or '0')).seconds / 60
+                tzsign = g[8]
+                tzhours = int(g[9] or 0)
+                tzminutes = int(g[10] or 0)
+                if not (0 <= tzhours < 24 and 0 <= tzminutes < 60):
+                    return None
+                tz = tzhours * 60 + tzminutes
                 if tz == 0:
                     tzinfo = utc
                 else:
                     tzinfo = FixedOffset(-tz if tzsign == '-' else tz,
-                                         '%s%s:%s' %
-                                         (tzsign, tzhours, tzminutes))
+                                         '%s%02d:%02d' % (tzsign, tzhours,
+                                                          tzminutes))
             tm = [int(x) for x in (years, months, days,
                                    hours, minutes, seconds, useconds)]
             t = tzinfo.localize(datetime(*tm))
             return tzinfo.normalize(t)
-        except ValueError:
+        except (ValueError, OverflowError):
             pass
 
     return None
 
+def _libc_parse_date(text, tzinfo):
+    for format in ('%x %X', '%x, %X', '%X %x', '%X, %x', '%x', '%c',
+                   '%b %d, %Y'):
+        try:
+            tm = time.strptime(text, format)
+            dt = tzinfo.localize(datetime(*tm[0:6]))
+            return tzinfo.normalize(dt)
+        except (ValueError, OverflowError):
+            continue
+    try:
+        return _i18n_parse_date(text, tzinfo, None)
+    except (ValueError, OverflowError):
+        pass
+    return
+
 def parse_date(text, tzinfo=None, locale=None, hint='date'):
     tzinfo = tzinfo or localtz
     text = text.strip()
@@ -446,29 +605,33 @@ def parse_date(text, tzinfo=None, locale=None, hint='date'):
         if babel and locale:
             dt = _i18n_parse_date(text, tzinfo, locale)
         else:
-            for format in ['%x %X', '%x, %X', '%X %x', '%X, %x', '%x', '%c',
-                           '%b %d, %Y']:
-                try:
-                    tm = time.strptime(text, format)
-                    dt = tzinfo.localize(datetime(*tm[0:6]))
-                    dt = tzinfo.normalize(dt)
-                    break
-                except ValueError:
-                    continue
+            dt = _libc_parse_date(text, tzinfo)
     if dt is None:
         dt = _parse_relative_time(text, tzinfo)
     if dt is None:
-        hint = {'datetime': get_datetime_format_hint,
-                'date': get_date_format_hint
-               }.get(hint, lambda(l): hint)(locale)
-        raise TracError(_('"%(date)s" is an invalid date, or the date format '
-                          'is not known. Try "%(hint)s" instead.',
-                          date=text, hint=hint), _('Invalid Date'))
+        formatted_hint = {
+            'datetime': get_datetime_format_hint,
+            'date': get_date_format_hint,
+            'iso8601': lambda l: get_datetime_format_hint('iso8601'),
+        }.get(hint, lambda(l): hint)(locale)
+        if locale == 'iso8601' and hint in ('date', 'datetime') or \
+                hint == 'iso8601':
+            msg = _('"%(date)s" is an invalid date, or the date format '
+                    'is not known. Try "%(hint)s" instead.',
+                    date=text, hint=formatted_hint)
+        else:
+            isohint = get_date_format_hint('iso8601') \
+                      if hint == 'date' \
+                      else get_datetime_format_hint('iso8601')
+            msg = _('"%(date)s" is an invalid date, or the date format is '
+                    'not known. Try "%(hint)s" or "%(isohint)s" instead.',
+                    date=text, hint=formatted_hint, isohint=isohint)
+        raise TracError(msg, _('Invalid Date'))
     # Make sure we can convert it to a timestamp and back - fromtimestamp()
     # may raise ValueError if larger than platform C localtime() or gmtime()
     try:
         datetime.utcfromtimestamp(to_timestamp(dt))
-    except ValueError:
+    except (ValueError, OverflowError):
         raise TracError(_('The date "%(date)s" is outside valid range. '
                           'Try a date closer to present time.', date=text),
                           _('Invalid Date'))
@@ -483,15 +646,17 @@ def _i18n_parse_date_pattern(locale):
         'm': ('m',),
         's': ('s',),
     }
-    regexp = [r'[0-9]+']
 
-    date_format = get_date_format('medium', locale=locale)
-    time_format = get_time_format('medium', locale=locale)
-    datetime_format = get_datetime_format('medium', locale=locale)
-    formats = (
-        datetime_format.replace('{0}', time_format.format) \
-                       .replace('{1}', date_format.format),
-        date_format.format)
+    if locale is None:
+        formats = (_libc_get_datetime_format_hint(format=True),
+                   _libc_get_date_format_hint(format=True))
+    else:
+        date_format = get_date_format('medium', locale=locale)
+        time_format = get_time_format('medium', locale=locale)
+        datetime_format = get_datetime_format('medium', locale=locale)
+        formats = (datetime_format.replace('{0}', time_format.format) \
+                                  .replace('{1}', date_format.format),
+                   date_format.format)
 
     orders = []
     for format in formats:
@@ -503,49 +668,64 @@ def _i18n_parse_date_pattern(locale):
                     order.append((idx, key))
                     break
         order.sort()
-        order = dict((key, idx) for idx, (_, key) in enumerate(order))
-        orders.append(order)
-
-    month_names = {
-        'jan': 1, 'feb': 2, 'mar': 3, 'apr': 4, 'may': 5, 'jun': 6,
-        'jul': 7, 'aug': 8, 'sep': 9, 'oct': 10, 'nov': 11, 'dec': 12,
-    }
-    if formats[0].find('%(MMM)s') != -1:
-        for width in ('wide', 'abbreviated'):
-            names = get_month_names(width, locale=locale)
-            for num, name in names.iteritems():
-                name = name.lower()
-                month_names[name] = num
-    regexp.extend(month_names.iterkeys())
+        orders.append(dict((key, idx) for idx, (_, key) in enumerate(order)))
 
+    # always allow using English names regardless of locale
+    month_names = dict(zip(('jan', 'feb', 'mar', 'apr', 'may', 'jun',
+                            'jul', 'aug', 'sep', 'oct', 'nov', 'dec',),
+                           xrange(1, 13)))
     period_names = {'am': 'am', 'pm': 'pm'}
-    if formats[0].find('%(a)s') != -1:
-        names = get_period_names(locale=locale)
-        for period, name in names.iteritems():
-            name = name.lower()
-            period_names[name] = period
-    regexp.extend(period_names.iterkeys())
+
+    if locale is None:
+        for num in xrange(1, 13):
+            t = datetime(1999, num, 1, tzinfo=utc)
+            names = format_date(t, '%b\t%B', utc).split('\t')
+            month_names.update((name.lower(), num) for name in names
+                               if str(num) not in name)
+        for num, period in ((11, 'am'), (23, 'pm')):
+            t = datetime(1999, 1, 1, num, tzinfo=utc)
+            name = format_datetime(t, '%p', utc)
+            if name:
+                period_names[name.lower()] = period
+    else:
+        if formats[0].find('%(MMM)s') != -1:
+            for width in ('wide', 'abbreviated'):
+                names = get_month_names(width, locale=locale)
+                month_names.update((name.lower(), num)
+                                   for num, name in names.iteritems())
+        if formats[0].find('%(a)s') != -1:
+            names = get_period_names(locale=locale)
+            period_names.update((name.lower(), period)
+                                for period, name in names.iteritems()
+                                if period in ('am', 'pm'))
+
+    regexp = ['[0-9]+']
+    regexp.extend(re.escape(name) for name in month_names)
+    regexp.extend(re.escape(name) for name in period_names)
 
     return {
         'orders': orders,
-        'regexp': re.compile('(%s)' % '|'.join(regexp),
-                             re.IGNORECASE | re.UNICODE),
+        'regexp': re.compile('(%s)' % '|'.join(regexp), re.IGNORECASE),
         'month_names': month_names,
         'period_names': period_names,
     }
 
-_I18N_PARSE_DATE_PATTERNS = dict(map(lambda l: (l, False),
-                                     get_available_locales()))
+_I18N_PARSE_DATE_PATTERNS = {}
+_I18N_PARSE_DATE_PATTERNS_LIBC = {}
 
 def _i18n_parse_date(text, tzinfo, locale):
-    locale = Locale.parse(locale)
-    key = str(locale)
-    pattern = _I18N_PARSE_DATE_PATTERNS.get(key)
-    if pattern is False:
-        pattern = _i18n_parse_date_pattern(locale)
-        _I18N_PARSE_DATE_PATTERNS[key] = pattern
+    if locale is None:
+        key = getlocale(LC_TIME)[0]
+        patterns = _I18N_PARSE_DATE_PATTERNS_LIBC
+    else:
+        locale = Locale.parse(locale)
+        key = str(locale)
+        patterns = _I18N_PARSE_DATE_PATTERNS
+
+    pattern = patterns.get(key)
     if pattern is None:
-        return None
+        pattern = _i18n_parse_date_pattern(locale)
+        patterns[key] = pattern
 
     regexp = pattern['regexp']
     period_names = pattern['period_names']
@@ -555,7 +735,7 @@ def _i18n_parse_date(text, tzinfo, locale):
         try:
             return _i18n_parse_date_0(text, order, regexp, period_names,
                                       month_names, tzinfo)
-        except ValueError:
+        except (ValueError, OverflowError):
             continue
 
     return None
@@ -648,7 +828,7 @@ _time_starts = dict(
 
 def _parse_relative_time(text, tzinfo, now=None):
     if now is None:     # now argument for unit tests
-        now = datetime.now(tzinfo)
+        now = datetime_now(tzinfo)
     if text == 'now':
         return now
 
@@ -749,67 +929,103 @@ class LocalTimezone(tzinfo):
 
     @classmethod
     def _initialize(cls):
-        cls._std_tz = cls(False)
         cls._std_offset = timedelta(seconds=-time.timezone)
+        cls._std_tz = cls(cls._std_offset)
         if time.daylight:
-            cls._dst_tz = cls(True)
             cls._dst_offset = timedelta(seconds=-time.altzone)
+            cls._dst_tz = cls(cls._dst_offset)
         else:
-            cls._dst_tz = cls._std_tz
             cls._dst_offset = cls._std_offset
+            cls._dst_tz = cls._std_tz
         cls._dst_diff = cls._dst_offset - cls._std_offset
 
-    def __init__(self, is_dst=None):
-        self.is_dst = is_dst
+    def __init__(self, offset=None):
+        self._offset = offset
 
     def __str__(self):
-        offset = self.utcoffset(datetime.now())
-        secs = offset.days * 3600 * 24 + offset.seconds
-        hours, rem = divmod(abs(secs), 3600)
-        return 'UTC%c%02d:%02d' % ('-' if secs < 0 else '+', hours, rem / 60)
+        return self._tzname_offset(self.utcoffset(datetime_now()))
 
     def __repr__(self):
-        if self.is_dst is None:
+        if self._offset is None:
             return '<LocalTimezone "%s" %s "%s" %s>' % \
                    (time.tzname[False], self._std_offset,
                     time.tzname[True], self._dst_offset)
-        if self.is_dst:
-            offset = self._dst_offset
+        return '<LocalTimezone "%s" %s>' % (self._tzname(), self._offset)
+
+    def _tzname(self):
+        if self is self._std_tz:
+            return time.tzname[False]
+        elif self is self._dst_tz:
+            return time.tzname[True]
+        elif self._offset is not None:
+            return self._tzname_offset(self._offset)
         else:
-            offset = self._std_offset
-        return '<LocalTimezone "%s" %s>' % (time.tzname[self.is_dst], offset)
+            return '%s, %s' % time.tzname
+
+    def _tzname_offset(self, offset):
+        secs = offset.days * 3600 * 24 + offset.seconds
+        hours, rem = divmod(abs(secs), 3600)
+        return 'UTC%c%02d:%02d' % ('+-'[secs < 0], hours, rem / 60)
+
+    def _tzinfo(self, dt, is_dst=False):
+        tzinfo = dt.tzinfo
+        if isinstance(tzinfo, LocalTimezone) and tzinfo._offset is not None:
+            return tzinfo
+
+        base_tt = (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second,
+                   dt.weekday(), 0)
+        local_tt = [None, None]
+        for idx in (0, 1):
+            try:
+                local_tt[idx] = time.localtime(time.mktime(base_tt + (idx,)))
+            except (ValueError, OverflowError):
+                pass
+        if local_tt[0] is local_tt[1] is None:
+            return self._std_tz
+
+        std_correct = local_tt[0] and local_tt[0].tm_isdst == 0
+        dst_correct = local_tt[1] and local_tt[1].tm_isdst == 1
+        if is_dst is None and std_correct is dst_correct:
+            if std_correct:
+                raise ValueError('Ambiguous time "%s"' % dt)
+            if not std_correct:
+                raise ValueError('Non existent time "%s"' % dt)
+        tt = None
+        if std_correct is dst_correct is True:
+            tt = local_tt[bool(is_dst)]
+        elif std_correct is True:
+            tt = local_tt[0]
+        elif dst_correct is True:
+            tt = local_tt[1]
+        if tt:
+            utc_ts = to_timestamp(datetime(tzinfo=utc, *tt[:6]))
+            tz_offset = timedelta(seconds=utc_ts - time.mktime(tt))
+        else:
+            dt = dt.replace(tzinfo=utc)
+            utc_ts = to_timestamp(dt)
+            dt -= timedelta(hours=6)
+            tt = (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second,
+                  dt.weekday(), 0, -1)
+            tz_offset = timedelta(seconds=utc_ts - time.mktime(tt) - 6 * 3600)
+
+        # if UTC offset doesn't match timezone offset, create a
+        # LocalTimezone instance with the UTC offset (#11563)
+        if tz_offset == self._std_offset:
+            tz = self._std_tz
+        elif tz_offset == self._dst_offset:
+            tz = self._dst_tz
+        else:
+            tz = LocalTimezone(tz_offset)
+        return tz
 
     def _is_dst(self, dt, is_dst=False):
-        if self.is_dst is not None:
-            return self.is_dst
-
-        tt = (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second,
-              dt.weekday(), 0)
-        try:
-            std_tt = time.localtime(time.mktime(tt + (0,)))
-            dst_tt = time.localtime(time.mktime(tt + (1,)))
-        except (ValueError, OverflowError):
-            return False
-
-        std_correct = std_tt.tm_isdst == 0
-        dst_correct = dst_tt.tm_isdst == 1
-        if std_correct is dst_correct:
-            if is_dst is None:
-                if std_correct is True:
-                    raise ValueError('Ambiguous time "%s"' % dt)
-                if std_correct is False:
-                    raise ValueError('Non existent time "%s"' % dt)
-            return is_dst
-        if std_correct:
-            return False
-        if dst_correct:
+        tz = self._tzinfo(dt, is_dst)
+        if tz is self._dst_tz:
             return True
+        return False
 
     def utcoffset(self, dt):
-        if self._is_dst(dt):
-            return self._dst_offset
-        else:
-            return self._std_offset
+        return self._tzinfo(dt)._offset
 
     def dst(self, dt):
         if self._is_dst(dt):
@@ -818,16 +1034,12 @@ class LocalTimezone(tzinfo):
             return _zero
 
     def tzname(self, dt):
-        return time.tzname[self._is_dst(dt)]
+        return self._tzinfo(dt)._tzname()
 
     def localize(self, dt, is_dst=False):
         if dt.tzinfo is not None:
             raise ValueError('Not naive datetime (tzinfo is already set)')
-        if self._is_dst(dt, is_dst):
-            tz = self._dst_tz
-        else:
-            tz = self._std_tz
-        return dt.replace(tzinfo=tz)
+        return dt.replace(tzinfo=self._tzinfo(dt, is_dst))
 
     def normalize(self, dt, is_dst=False):
         if dt.tzinfo is None:
@@ -839,12 +1051,22 @@ class LocalTimezone(tzinfo):
     def fromutc(self, dt):
         if dt.tzinfo is None or dt.tzinfo is not self:
             raise ValueError('fromutc: dt.tzinfo is not self')
-        tt = time.localtime(to_timestamp(dt.replace(tzinfo=utc)))
-        if tt.tm_isdst > 0:
+        dt = dt.replace(tzinfo=utc)
+        try:
+            tt = time.localtime(to_timestamp(dt))
+        except (ValueError, OverflowError):
+            return dt.replace(tzinfo=self._std_tz) + self._std_offset
+        # if UTC offset from localtime() doesn't match timezone offset,
+        # create a LocalTimezone instance with the UTC offset (#11563)
+        new_dt = datetime(*(tt[:6] + (dt.microsecond, utc)))
+        tz_offset = new_dt - dt
+        if tz_offset == self._std_offset:
+            tz = self._std_tz
+        elif tz_offset == self._dst_offset:
             tz = self._dst_tz
         else:
-            tz = self._std_tz
-        return datetime(microsecond=dt.microsecond, tzinfo=tz, *tt[0:6])
+            tz = LocalTimezone(tz_offset)
+        return new_dt.replace(tzinfo=tz)
 
 
 utc = FixedOffset(0, 'UTC')
@@ -880,7 +1102,8 @@ _timezones = [
     FixedOffset(360, 'GMT +6:00'),   FixedOffset(420, 'GMT +7:00'),
     FixedOffset(480, 'GMT +8:00'),   FixedOffset(540, 'GMT +9:00'),
     FixedOffset(600, 'GMT +10:00'),  FixedOffset(660, 'GMT +11:00'),
-    FixedOffset(720, 'GMT +12:00'),  FixedOffset(780, 'GMT +13:00')]
+    FixedOffset(720, 'GMT +12:00'),  FixedOffset(780, 'GMT +13:00'),
+    FixedOffset(840, 'GMT +14:00')]
 _tzmap = dict([(z.zone, z) for z in _timezones])
 
 all_timezones = [z.zone for z in _timezones]
