@@ -61,6 +61,13 @@ def to_unicode(text, charset=None):
         except UnicodeDecodeError:
             return unicode(text, 'latin1')
     elif isinstance(text, Exception):
+        if os.name == 'nt' and isinstance(text, (OSError, IOError)):
+            # the exception might have a localized error string encoded with
+            # ANSI codepage if OSError and IOError on Windows
+            try:
+                return unicode(str(text), 'mbcs')
+            except UnicodeError:
+                pass
         # two possibilities for storing unicode strings in exception data:
         try:
             # custom __str__ method on the exception (e.g. PermissionError)
@@ -131,10 +138,10 @@ def strip_line_ws(text, leading=True, trailing=True):
 
 _js_quote = {'\\': '\\\\', '"': '\\"', '\b': '\\b', '\f': '\\f',
              '\n': '\\n', '\r': '\\r', '\t': '\\t', "'": "\\'"}
-for i in range(0x20) + [ord(c) for c in '&<>']:
-    _js_quote.setdefault(chr(i), '\\u%04x' % i)
-_js_quote_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t\'&<>]')
-_js_string_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t&<>]')
+for i in range(0x20) + [ord(c) for c in u'&<>\u2028\u2029']:
+    _js_quote.setdefault(unichr(i), '\\u%04x' % i)
+_js_quote_re = re.compile(ur'[\x00-\x1f\\"\b\f\n\r\t\'&<>\u2028\u2029]')
+_js_string_re = re.compile(ur'[\x00-\x1f\\"\b\f\n\r\t&<>\u2028\u2029]')
 
 
 def javascript_quote(text):
@@ -221,24 +228,27 @@ def quote_query_string(text):
 
 
 def to_utf8(text, charset='latin1'):
-    """Convert a string to UTF-8, assuming the encoding is either UTF-8, ISO
-    Latin-1, or as specified by the optional `charset` parameter.
+    """Convert a string to an UTF-8 `str` object.
 
-    .. deprecated :: 0.10
-       You should use `unicode` strings only.
+    If the input is not an `unicode` object, we assume the encoding is
+    already UTF-8, ISO Latin-1, or as specified by the optional
+    *charset* parameter.
     """
-    try:
-        # Do nothing if it's already utf-8
-        u = unicode(text, 'utf-8')
-        return text
-    except UnicodeError:
+    if isinstance(text, unicode):
+        u = text
+    else:
         try:
-            # Use the user supplied charset if possible
-            u = unicode(text, charset)
+            # Do nothing if it's already utf-8
+            u = unicode(text, 'utf-8')
+            return text
         except UnicodeError:
-            # This should always work
-            u = unicode(text, 'latin1')
-        return u.encode('utf-8')
+            try:
+                # Use the user supplied charset if possible
+                u = unicode(text, charset)
+            except UnicodeError:
+                # This should always work
+                u = unicode(text, 'latin1')
+    return u.encode('utf-8')
 
 
 class unicode_passwd(unicode):
@@ -317,26 +327,32 @@ def text_width(text, ambiwidth=1):
     return sum([2 if east_asian_width(chr) in twice else 1
                 for chr in to_unicode(text)])
 
-_default_ambiwidth = 1  # Default width of East Asian Ambiguous (A)
-if os.name == 'nt':
-    try:
-        # `ctypes` is available since Python 2.5
+
+def _get_default_ambiwidth():
+    """Return width of East Asian Ambiguous based on locale environment
+    variables or Windows codepage.
+    """
+
+    if os.name == 'nt':
         import ctypes
         codepage = ctypes.windll.kernel32.GetConsoleOutputCP()
-    except ImportError:
-        # Try to retrieve the codepage from stderr and stdout
-        codepage = (sys.stderr.encoding or sys.stdout.encoding or '')[2:]
-        codepage = codepage.isdigit() and int(codepage) or 0
+        if codepage in (932,   # Japanese (Shift-JIS)
+                        936,   # Chinese Simplified (GB2312)
+                        949,   # Korean (Unified Hangul Code)
+                        950):  # Chinese Traditional (Big5)
+            return 2
+    else:
+        for name in ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG'):
+            value = os.environ.get(name) or ''
+            if value:
+                if name == 'LANGUAGE' and ':' in value:
+                    value = value.split(':')[0]
+                return 2 if value.lower().startswith(('zh', 'ja', 'ko')) else 1
 
-    if codepage in (932,  # Japanese (Shift-JIS)
-                    936,  # Chinese Simplified (GB2312)
-                    949,  # Korean (Unified Hangul Code)
-                    950): # Chinese Traditional (Big5)
-        _default_ambiwidth = 2
-    del codepage
-else:
-    if re.match(r'zh|ja|kr', os.environ.get('LANG') or '', re.IGNORECASE):
-        _default_ambiwidth = 2
+    return 1
+
+
+_default_ambiwidth = _get_default_ambiwidth()
 
 
 def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
@@ -372,8 +388,23 @@ def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
     def tw(text):
         return text_width(text, ambiwidth=ambiwidth)
 
-    # Convert each cell to an unicode object
-    data = [[to_text(cell) for cell in row] for row in data]
+    def to_lines(data):
+        lines = []
+        for row in data:
+            row = [to_text(cell) for cell in row]
+            if any('\n' in cell for cell in row):
+                row = [cell.splitlines() for cell in row]
+                max_lines = max(len(cell) for cell in row)
+                for cell in row:
+                    if len(cell) < max_lines:
+                        cell += [''] * (max_lines - len(cell))
+                lines.extend([cell[idx] for cell in row]
+                             for idx in xrange(max_lines))
+            else:
+                lines.append(row)
+        return lines
+
+    data = to_lines(data)
 
     num_cols = len(data[0])
     col_width = [max(tw(row[idx]) for row in data)
@@ -382,15 +413,15 @@ def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
     out.write('\n')
     for ridx, row in enumerate(data):
         for cidx, cell in enumerate(row):
-            if headers and ridx == 0:
-                sp = '%*s' % (tw(sep), ' ') # No separator in header
-            else:
-                sp = sep
             if cidx + 1 == num_cols:
-                sp = '' # No separator after last column
-
-            line = u'%-*s%s' % (col_width[cidx] - tw(cell) + len(cell),
-                                cell, sp)
+                line = cell  # No separator after last column
+            else:
+                if headers and ridx == 0:
+                    sp = ' ' * tw(sep)  # No separator in header
+                else:
+                    sp = sep
+                line = u'%-*s%s' % (col_width[cidx] - tw(cell) + len(cell),
+                                    cell, sp)
             line = line.encode(charset, 'replace')
             out.write(line)
 
@@ -402,17 +433,19 @@ def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
 
 
 def shorten_line(text, maxlen=75):
-    """Truncates content to at most `maxlen` characters.
+    """Truncates `text` to length less than or equal to `maxlen` characters.
 
     This tries to be (a bit) clever and attempts to find a proper word
     boundary for doing so.
     """
-    if len(text or '') < maxlen:
+    if len(text or '') <= maxlen:
         return text
-    cut = max(text.rfind(' ', 0, maxlen), text.rfind('\n', 0, maxlen))
+    suffix = ' ...'
+    maxtextlen = maxlen - len(suffix)
+    cut = max(text.rfind(' ', 0, maxtextlen), text.rfind('\n', 0, maxtextlen))
     if cut < 0:
-        cut = maxlen
-    return text[:cut] + ' ...'
+        cut = maxtextlen
+    return text[:cut] + suffix
 
 
 class UnicodeTextWrapper(textwrap.TextWrapper):
