@@ -22,7 +22,7 @@ import trac.tests.compat
 from genshi.builder import tag
 from trac import perm
 from trac.core import TracError
-from trac.test import EnvironmentStub, Mock, MockPerm, locale_en
+from trac.test import EnvironmentStub, Mock, MockPerm
 from trac.util import create_file
 from trac.util.datefmt import utc
 from trac.util.text import shorten_line
@@ -50,17 +50,7 @@ class RequestHandlerPermissionsTestCaseBase(unittest.TestCase):
         self.req_handler = module_class(self.env)
 
     def tearDown(self):
-        self.env.reset_db()
-        shutil.rmtree(self.path)
-
-    def create_request(self, authname='anonymous', **kwargs):
-        kw = {'perm': perm.PermissionCache(self.env, authname), 'args': {},
-              'href': self.env.href, 'abs_href': self.env.abs_href,
-              'tz': utc, 'locale': None, 'lc_time': locale_en,
-              'authname': authname, 'chrome': {'notices': [], 'warnings': []},
-              'method': None, 'get_header': lambda v: None}
-        kw.update(kwargs)
-        return Mock(**kw)
+        self.env.reset_db_and_disk()
 
     def get_navigation_items(self, req):
         return self.req_handler.get_navigation_items(req)
@@ -180,6 +170,55 @@ class RequestTestCase(unittest.TestCase):
         self.assertEqual('http://example.com/trac/test',
                          headers_sent['Location'])
 
+    def test_redirect_with_post_and_hash_for_msie(self):
+        url = 'http://example.com/trac/ticket/1#comment:2'
+        msie303 = 'http://example.com/trac/ticket/1#__msie303:comment:2'
+
+        def location(ua):
+            status_sent = []
+            headers_sent = {}
+            def start_response(status, headers):
+                status_sent.append(status)
+                headers_sent.update(dict(headers))
+            environ = self._make_environ(method='POST', HTTP_USER_AGENT=ua)
+            req = Request(environ, start_response,)
+            req.session = Mock(save=lambda: None)
+            self.assertRaises(RequestDone, req.redirect, url)
+            self.assertEqual('303 See Other', status_sent[0])
+            return headers_sent['Location']
+
+        # IE 11 strict mode
+        self.assertEqual(url, location(
+            'Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko'))
+        # IE 11 compatibility view mode
+        self.assertEqual(url, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/7.0)'))
+        # IE 10 strict mode
+        self.assertEqual(url, location(
+            'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)'
+            ))
+        # IE 10 compatibility view mode
+        self.assertEqual(url, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0)'))
+        # IE 9 strict mode
+        self.assertEqual(msie303, location(
+            'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)'))
+        # IE 9 compatibility view mode
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0)'))
+        # IE 8 strict mode
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)'))
+        # IE 8 compatibility view mode
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)'))
+        # IE 7
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)'))
+        # IE 6
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)'))
+
     def test_write_iterable(self):
         buf = StringIO()
         def write(data):
@@ -255,6 +294,112 @@ class RequestTestCase(unittest.TestCase):
         req = Request(environ, None)
         self.assertEqual('test', req.read(size=4))
 
+    def _test_qs_with_null_bytes(self, environ):
+        req = Request(environ, None)
+        try:
+            req.args['action']
+        except HTTPBadRequest, e:
+            self.assertEqual("400 Bad Request (Invalid request arguments.)",
+                             unicode(e))
+        else:
+            self.fail("HTTPBadRequest not raised.")
+
+    def test_qs_with_null_bytes_for_name(self):
+        environ = self._make_environ(method='GET',
+                                     **{'QUERY_STRING': 'acti\x00n=fOO'})
+        self._test_qs_with_null_bytes(environ)
+
+    def test_qs_with_null_bytes_for_value(self):
+        environ = self._make_environ(method='GET',
+                                     **{'QUERY_STRING': 'action=f\x00O'})
+        self._test_qs_with_null_bytes(environ)
+
+    def test_post_with_unnamed_value(self):
+        boundary = '_BOUNDARY_'
+        form_data = """\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="foo"\r\n\
+\r\n\
+named value\r\n\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name=""\r\n\
+\r\n\
+name is empty\r\n\
+--%(boundary)s\r\n\
+Content-Disposition: form-data\r\n\
+\r\n\
+unnamed value\r\n\
+--%(boundary)s--\r\n\
+"""
+        form_data %= {'boundary': boundary}
+        content_type = 'multipart/form-data; boundary="%s"' % boundary
+        environ = self._make_environ(method='POST', **{
+            'wsgi.input': StringIO(form_data),
+            'CONTENT_LENGTH': str(len(form_data)),
+            'CONTENT_TYPE': content_type
+        })
+        req = Request(environ, None)
+
+        self.assertEqual('named value', req.args['foo'])
+        self.assertEqual([('foo', 'named value'), ('', 'name is empty'),
+                          (None, 'unnamed value')], req.arg_list)
+
+    def _test_post_with_null_bytes(self, form_data):
+        boundary = '_BOUNDARY_'
+        content_type = 'multipart/form-data; boundary="%s"' % boundary
+        form_data %= {'boundary': boundary}
+
+        environ = self._make_environ(method='POST', **{
+            'wsgi.input': StringIO(form_data),
+            'CONTENT_LENGTH': str(len(form_data)),
+            'CONTENT_TYPE': content_type
+        })
+        req = Request(environ, None)
+
+        try:
+            req.args['action']
+        except HTTPBadRequest, e:
+            self.assertEqual("400 Bad Request (Invalid request arguments.)",
+                             unicode(e))
+        else:
+            self.fail("HTTPBadRequest not raised.")
+
+    def test_post_with_null_bytes_for_filename(self):
+        form_data = """\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="attachment"; filename="thefi\x00le.txt"\r\n\
+Content-Type: text/plain\r\n\
+\r\n\
+The file content.\r\n\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="action"\r\n\
+\r\n\
+new\r\n\
+--%(boundary)s--\r\n\
+"""
+        self._test_post_with_null_bytes(form_data)
+
+    def test_post_with_null_bytes_for_name(self):
+        form_data = """\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="acti\x00n"\r\n\
+\r\n\
+new\r\n\
+--%(boundary)s--\r\n\
+"""
+
+        self._test_post_with_null_bytes(form_data)
+
+    def test_post_with_null_bytes_for_value(self):
+        form_data = """\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="action"\r\n\
+\r\n\
+ne\x00w\r\n\
+--%(boundary)s--\r\n\
+"""
+        self._test_post_with_null_bytes(form_data)
+
     def test_qs_on_post(self):
         """Make sure req.args parsing is consistent even after the backwards
         incompatible change introduced in Python 2.6.
