@@ -31,15 +31,17 @@ from trac.resource import (
     Resource, ResourceNotFound, get_resource_name, get_resource_summary,
     get_resource_url
 )
+from trac.util import as_int
 from trac.util.datefmt import format_date, from_utimestamp, user_time
 from trac.util.html import escape, find_element
 from trac.util.presentation import separated
 from trac.util.text import unicode_quote, to_unicode, stripws
-from trac.util.translation import _, dgettext, cleandoc_
+from trac.util.translation import _, dgettext, cleandoc_, tag_
 from trac.web.chrome import chrome_resource_path
 from trac.wiki.api import IWikiMacroProvider, WikiSystem, parse_args
 from trac.wiki.formatter import (
-    format_to_html, format_to_oneliner, extract_link, OutlineFormatter
+    MacroError, OutlineFormatter, extract_link, format_to_html,
+    format_to_oneliner, system_message
 )
 
 
@@ -119,9 +121,9 @@ class TitleIndexMacro(WikiMacroBase):
         args, kw = parse_args(content)
         prefix = args[0].strip() if args else None
         hideprefix = args and len(args) > 1 and args[1].strip() == 'hideprefix'
-        minsize = max(int(kw.get('min', 1)), 1)
+        minsize = _arg_as_int(kw.get('min', 1), 'min', min=1)
         minsize_group = max(minsize, 2)
-        depth = int(kw.get('depth', -1))
+        depth = _arg_as_int(kw.get('depth', -1), 'depth', min=-1)
         start = prefix.count('/') if prefix else 0
         format = kw.get('format', '')
 
@@ -322,7 +324,7 @@ class RecentChangesMacro(WikiMacroBase):
     def expand_macro(self, formatter, name, content):
         args, kw = parse_args(content)
         prefix = args[0].strip() if args else None
-        limit = int(args[1].strip()) if len(args) > 1 else None
+        limit = _arg_as_int(args[1].strip(), min=1) if len(args) > 1 else None
         group = kw.get('group', 'date')
 
         sql = """SELECT name, max(version) AS max_version,
@@ -408,10 +410,12 @@ class PageOutlineMacro(WikiMacroBase):
             if len(argv) > 0:
                 depth = argv[0]
                 if '-' in depth:
-                    min_depth, max_depth = [int(d)
-                                            for d in depth.split('-', 1)]
+                    min_depth, max_depth = \
+                        [_arg_as_int(d, min=min_depth, max=max_depth)
+                         for d in depth.split('-', 1)]
                 else:
-                    min_depth = max_depth = int(depth)
+                    min_depth = max_depth = \
+                        _arg_as_int(depth, min=min_depth, max=max_depth)
                 if len(argv) > 1:
                     title = argv[1].strip()
                     for arg in argv[2:]:
@@ -510,7 +514,7 @@ class ImageMacro(WikiMacroBase):
 
     def is_inline(self, content):
         args = [stripws(arg) for arg
-                             in self._split_args_re.split(content)[1::2]]
+                             in self._split_args_re.split(content or '')[1::2]]
         return 'inline' in args
 
     _split_re = r'''((?:[^%s"']|"[^"]*"|'[^']*')+)'''
@@ -580,12 +584,12 @@ class ImageMacro(WikiMacroBase):
                             val in ('top', 'middle', 'bottom')):
                         args.append(val)
                     elif key in ('margin-top', 'margin-bottom'):
-                        style[key] = ' %dpx' % int(val)
+                        style[key] = ' %dpx' % _arg_as_int(val, key, min=1)
                     elif key in ('margin', 'margin-left', 'margin-right') \
                              and 'display' not in style:
-                        style[key] = ' %dpx' % int(val)
+                        style[key] = ' %dpx' % _arg_as_int(val, key, min=1)
                     elif key == 'border':
-                        style['border'] = ' %dpx solid' % int(val)
+                        style['border'] = ' %dpx solid' % _arg_as_int(val, key)
                     else:
                         m = self._quoted_re.search(val)  # unquote "..." and '...'
                         if m:
@@ -655,19 +659,24 @@ class ImageMacro(WikiMacroBase):
         elif len(parts) == 1:  # it's an attachment of the current resource
             attachment = formatter.resource.child('attachment', filespec)
         else:
-            raise TracError(_("No filespec given"))
-        if attachment and 'ATTACHMENT_VIEW' in formatter.perm(attachment):
-            url = get_resource_url(self.env, attachment, formatter.href)
-            raw_url = get_resource_url(self.env, attachment, formatter.href,
-                                       format='raw')
+            return system_message(_("No filespec given"))
+        if attachment:
             try:
                 desc = get_resource_summary(self.env, attachment)
             except ResourceNotFound:
+                link = None
                 raw_url = chrome_resource_path(formatter.context.req,
                                                'common/attachment.png')
                 desc = _('No image "%(id)s" attached to %(parent)s',
                          id=attachment.id,
                          parent=get_resource_name(self.env, attachment.parent))
+            else:
+                if 'ATTACHMENT_VIEW' in formatter.perm(attachment):
+                    url = get_resource_url(self.env, attachment,
+                                           formatter.href)
+                    raw_url = get_resource_url(self.env, attachment,
+                                               formatter.href, format='raw')
+
         for key in ('title', 'alt'):
             if desc and key not in attr:
                 attr[key] = desc
@@ -794,8 +803,8 @@ class TracIniMacro(WikiMacroBase):
              format_to_html(self.env, formatter.context, section_doc),
              tag.table(class_='wiki')(tag.tbody(
                  tag.tr(tag.td(tag.tt(option.name)),
-                        tag.td(format_to_oneliner(
-                            self.env, formatter.context, getdoc(option))),
+                        tag.td(format_to_html(self.env, formatter.context,
+                                              getdoc(option))),
                         default_cell(option),
                         class_='odd' if idx % 2 else 'even')
                  for idx, option in
@@ -848,9 +857,10 @@ class TracGuideTocMacro(WikiMacroBase):
     """Display a table of content for the Trac guide.
 
     This macro shows a quick and dirty way to make a table-of-contents
-    for the Help/Guide. The table of contents will contain the Trac* and
-    WikiFormatting pages, and can't be customized. Search for TocMacro for a
-    a more customizable table of contents.
+    for the !Help/Guide. The table of contents will contain the Trac* and
+    WikiFormatting pages, and can't be customized. See the
+    [https://trac-hacks.org/wiki/TocMacro TocMacro] for a more customizable
+    table of contents.
     """)
 
     TOC = [('TracGuide',                    'Index'),
@@ -897,3 +907,12 @@ class TracGuideTocMacro(WikiMacroBase):
                            class_=(prefix+ref == curpage and 'active'))
                     for ref, title in self.TOC]),
             class_='wiki-toc')
+
+
+def _arg_as_int(val, key=None, min=None, max=None):
+    int_val = as_int(val, None, min=min, max=max)
+    if int_val is None:
+        raise MacroError(tag_("Invalid macro argument %(expr)s",
+                              expr=tag.tt("%s=%s" % (key, val))
+                                   if key else tag.tt(val)))
+    return int_val
